
require 'nano/base.rb' unless $facets_without_base

module Facets
class << self

  # This simply subs terms for Ruby's operators
  # and removes any trailing ? ! or =.
  def op_esc_path( name )
    ri = name.rindex('/')
    if ri
      dir, file = name[0...ri], name[ri+1..-1]
    else
      dir, file = nil, name
    end
    file = op_esc(file.strip) #.gsub(OperatorsRegexp) { OperatorsEscTable[$1] }
    #file.chomp('?').chomp('!').chomp('=')
    dir ? File.join( dir, file ) : file
  end

  def op_esc( str )
    str = str.to_s.chomp('.rb')
    str = str.strip.gsub(OPERATORS_REGEXP){ OPERATORS_ESCAPE[$1] }
    str.chomp('?').chomp('!').chomp('=')
  end

  OPERATORS = %w{ +@ -@ + - ** * / % ~ <=> << >> < > === == =~ <= >= | & ^ []= [] }
  OPERATORS_REGEXP = Regexp.new( '(' << OPERATORS.collect{ |k| Regexp.escape(k) }.join('|') << ')' )
  OPERATORS_ESCAPE = {
     "+@"   => "op_plus_self",
     "-@"   => "op_minus_self",
     "+"    => "op_plus",
     "-"    => "op_minus",
     "**"   => "op_pow",
     "*"    => "op_mul",
     "/"    => "op_div",
     "%"    => "op_mod",
     "~"    => "op_tilde",
     "<=>"  => "op_cmp",
     "<<"   => "op_lshift",
     ">>"   => "op_rshift",
     "<"    => "op_lt",
     ">"    => "op_gt",
     "==="  => "op_case_eq",
     "=="   => "op_equal",
     "=~"   => "op_apply",
     "<="   => "op_lt_eq",
     ">="   => "op_gt_eq",
     "|"    => "op_or",
     "&"    => "op_and",
     "^"    => "op_xor",
     "[]="  => "op_store",
     "[]"   => "op_fetch"
  }

  REQDIR = 'nano'
  SETDIR = 'nano'

  # Require methods for given class/module.
  alias_method :_require, :require

  def require( name )
    #_require( "#{SETDIR}/#{name}" )
    _require( File.join( 'facet', name ) )
  end

  #def require( klass, *meths )
  #  provide_features( klass, *meths )
  #end

  def provide_features( klass, *meths )
    classname = klass.to_s.downcase
    loaded = true
    meths.each do |m|
      me = op_esc(m)
      begin
        r = _require( "#{REQDIR}/#{classname}/#{me}" )
      rescue LoadError => e
        if klass.ancestors[1]
          classname = klass.ancestors[1].name.downcase
          retry
        else
          raise e
        end
      end
      loaded = loaded && r
    end
    loaded
  end

  #--
  # NOTE: The Fixnum constants can't be autoloaded.
  # Nor can any pure method module. Fix?
  #++

  def autorequire
    # built-in
    autoload( 'CGI'        , 'cgi' )
    autoload( 'FCGI'       , 'fcgi' )
    autoload( 'FileUtils'  , 'fileutils' )
    autoload( 'OpenStruct' , 'ostruct' )
    autoload( 'YAML'       , 'yaml' )
    # mega classes
    autoload( 'ANSICode'       , 'mega/ansicode' )
    autoload( 'BBCode'         , 'mega/bbcode' )
    autoload( 'BinaryReader'   , 'mega/binaryreader' )
    autoload( 'BasicObject'    , 'mega/basicobject' )
    autoload( 'ByteOrder'      , 'mega/byteorder' )
    autoload( 'ClassMethods'   , 'mega/classmethods' )
    autoload( 'Constant'       , 'mega/constants' )
    autoload( 'Coroutine'      , 'mega/coroutine' )
    autoload( 'CrossCase'      , 'mega/crosscase' )
    autoload( 'EnumerableArgs' , 'mega/enumerable_args' )
    autoload( 'EnumeratedType' , 'mega/enumtype' )
    autoload( 'FileList'       , 'mega/expirable' )
    autoload( 'FloatString'    , 'mega/floatstring' )
    autoload( 'Functor'        , 'mega/functor' )
    autoload( 'HArray'         , 'mega/harray' )
    autoload( 'Heap'           , 'mega/heap' )
    autoload( 'Infinity'       , 'mega/infinity' )
    autoload( 'Interval'       , 'mega/interval' )
    autoload( 'IOReactor'      , 'mega/ioreactor' )
    autoload( 'IORedirect'     , 'mega/ioredirect' )
    autoload( 'LRUCache'       , 'mega/lru_cache' )
    autoload( 'MethodProbe'    , 'mega/methodprobe' )
    autoload( 'Multiton'       , 'mega/multiton' )
    autoload( 'Nack'           , 'mega/nack' )
    autoload( 'NilComparable'  , 'mega/nilcomparable' )
    autoload( 'NotCopyable'    , 'mega/notcopyable' )
    autoload( 'OpenStructable' , 'mega/ostructable' )
    autoload( 'OrderedHash'    , 'mega/ohash' )
    autoload( 'Dictionary'     , 'mega/dictionary' )  #?
    autoload( 'Pool'           , 'mega/pool' )
    autoload( 'PromoteSelf'    , 'mega/promote_self' )
    autoload( 'Reference'      , 'mega/reference' )
    autoload( 'Semaphore'      , 'mega/semaphore' )
    autoload( 'StackParser'    , 'mega/stackparser' )
    autoload( 'StaticHash'     , 'mega/statichash' )
    autoload( 'SuperStruct'    , 'mega/superstruct' )
    autoload( 'SyncHash'       , 'mega/synchash' )
    autoload( 'SyncArray'      , 'mega/syncarray' )
    autoload( 'SyncHash'       , 'mega/synchash' )
    autoload( 'TagIterator'    , 'mega/tagiter' )
    autoload( 'Timer'          , 'mega/timer' )
    autoload( 'TracePoint'     , 'mega/tracepoint' )
    autoload( 'Tuple'          , 'mega/tuple' )
    autoload( 'Uninheritable'  , 'mega/uninheritable' )
    autoload( 'Version'        , 'mega/version' )
  end

  alias  :autoload_classes :autorequire
  alias  :autoload_modules :autorequire

end
end


class Module

  # Generalize method aquiring features from a 
  # providing module. You can use this with
  # Facets. Instead of
  #
  #   require 'facet/string/humanize'
  #   require 'facet/string/last'
  #
  # do
  #
  #   String.use Facets, 'humanize', 'last='
  #
  def use( package, *meths )
    if String === package or Symbol === package
      package = constant(package)
    end
    package.provide_features( self, *meths )
  end

end




# scrap

# module Kernel
#
#   loc = [
#     File.join( File.dirname(__FILE__), 'facets/ext' ),
#     File.join( File.dirname(__FILE__), 'facets/set' ),
#     File.join( File.dirname(__FILE__), 'facets/set/general' ),
#     File.join( File.dirname(__FILE__), 'facets/set/special' ),
#     File.join( File.dirname(__FILE__), 'facets' )
#   ]
#
#   alias_method :require_without_facets, :require #req = instance_method(:require)
#
#   define_method( :require ) do |req, *args|
#
#     loaded = false
#     success = false
#
#     if md = %r{^(facets|facet|nano)[\/\\]}.match( req )
#       name = FacetSystem.op_esc_path( md.post_match ) + '.rb'
#       dir = loc.find{ |d| File.file?( File.join(d,name) ) }
#       if dir
#         loaded = require_without_facets( File.join( dir,name ), *args )
#         success = true
#       end
#     end
#
#     unless success
#       #req.bind(self).call( name, *args )
#       loaded = require_without_facets( req, *args )
#     end
#
#     loaded
#   end
#
# end
