#--
# ClassMethods
#
# Copyright (c) 2005 Thomas Sawyer
#
# Ruby License
#
# This module is free software. You may use, modify, and/or redistribute this
# software under the same terms as Ruby.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.
#
# ==========================================================================
#  REVISION HISTORY
# ==========================================================================
#  2005.04.28  trans
#    Reimplemented for Mega based on Nabu's work.
# ==========================================================================
#
# CREDIT: Credit is due Nobu and Ulysses.
#
#++

#:title: ClassMethods
#
# When including modules, class/module methods are not inherited. Using this
# alternative version of append_features, you can easily do just that. Simply
# put all your class/module methods in a nested module called ClassMethods.
# Don’t forget to extend the original module, to make sure the nested module
# functions are still usable as module methods in the original.
#
#   module Mix
#     def inst_meth
#       puts 'inst_meth'
#     end
#
#     module ClassMethods
#       def class_meth
#         puts 'class_meth'
#       end
#     end
#
#     extend ClassMethods
#   end
#
#   class LesserClass
#     include Mix
#   end
#
#   LesserClass.class_meth  #=> class_meth
#

class Module
  alias_method :append_features_no_classmethods, :append_features
  def append_features(base)
    append_features_no_classmethods(base)
    begin clsm = self::ClassMethods
    rescue NameError
      clsm = nil
    end
    base.extend(clsm) if clsm
  end
end



#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#

# TODO

=begin testing

  require 'test/unit'

  # fixture

  module N
    module ClassMethods
      def n ; 43 ; end
    end

    extend ClassMethods
  end

  class X
    include N
    def n ; 11 ; end
  end

  # test

  class TC_ClassMethods
    def test_01
      assert_equal( 43, N.n )
    end
    def test_02
      assert_equal( 43, X.n )
    end
    def test_03
      assert_equal( 11, X.new.n )
    end
  end

=end
